﻿/* ***********************************************************************
The classes in the package edu.uriship.math.logic were created by

Barbara Kaskosz of the University of Rhode Island
(bkaskosz@math.uri.edu) and
Douglas E. Ensley of Shippensburg University
(deensl@ship.edu).

The materials are a part of the ongoing series 
"Flash Tools for Developers" by the authors.

This work is supported in part by the National Science Foundation
under the grant DUE-0535327.

Feel free to use and modify the templates and the classes to create
your own Flash applications for educational purposes under the
following two conditions:

1) Please include a line acknowledging this work and the authors
with your application. 

2) If you choose to modify the classes in the package 
edu.uriship.math.logic, put them into a package different from
the original. In this case too, please acknowledge this work.

Last modified: July 18, 2006

************************************************************************ */


import edu.uriship.math.logic.CompiledObject;


class  edu.uriship.math.logic.TruthParser {

  private  var f1Array:Array=["N"];
  private  var f2Array:Array=["I","O","A","B"];
    
  private var iserror:Number;
  private var errorMes:String;	

	
  private var tokenvalue:String;
  private var tokentype:String;
  private var tokenlength:Number;

  private var lets:String="abcdefghijklmnopqrstuwvxzy";
  
  private var opers:String=")(";
  
  private var connects:String=">=+-*";
  
  private var funs:String="NOABI";
 
  private var aVarNames;
	
  public function TruthParser(varArray:Array){
		
		if(varArray==undefined){
			
			this.aVarNames=[];
			
			} else {
		
		this.aVarNames=varArray;
		
			}
		
		this.iserror=0;
	
	    this.errorMes="";
		
		checkVarsNames(aVarNames);
		
	}
	
	
	
private function checkVarsNames(a:Array):Void {
	
	var k:Number;
	
	for(k=0;k<aVarNames.length;k++){
		
		if(!(aVarNames[k].length==1) || !(lets.indexOf(aVarNames[k])>-1)){
			
			trace("Dear programmer! Only one-letter variables are allowed and they must be entered as strings. Check your truth parser's contructor, variable number "+String(k+1)+".");
			
			return;
			
		}
		
	}
	
	
}
	

public function doCompile(inputstring:String):CompiledObject {
	
	var coCompObj:CompiledObject=new CompiledObject();
	
	var stepString:String;
	
	var conString:String;
	
	var fstack:Array=[];
	
	iserror=0;
	
	errorMes="";
		
	if(inputstring==undefined){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes="Undefined input.";
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
		
	}
		
	stepString=whiteSpaces(inputstring);
	
	if(stepString==""){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes="Empty input.";
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
		
	}
	
	checkLegal(stepString);
	
	if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
		
		}
	
	stepString=changeNotation(stepString);
	
	checkLegalTwo(stepString);
	
	if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
		
		}
		
			
	checkPars(stepString);
	
	if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
				
		}
		
		stepString=fixNotExpr(stepString);
		
		if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
				
		}
		
		checkToks(stepString);
		
		if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
				
		}
		
		checkImpliesExpr(stepString);
		
		if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
				
		}
		
		checkOrExpr(stepString);
		
		if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
				
		}
		
		checkAndExpr(stepString);
		
		if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
				
		}
		
		checkIffExpr(stepString);
		
		if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
				
		}
		
	stepString=conUnary(stepString);
			
	conString=conOper(conOper(conOper(conOper(stepString,"*"),"+"),">"),"=");
    
	if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
		
		}
		
	
	fstack=makeStack(conString);
	
	if(iserror==1){
		
		coCompObj.PolishArray=[];
		
		coCompObj.errorMes=errorMes;
		
		coCompObj.errorStatus=1;
		
		return coCompObj;
			
		}
	
	else {
		
		coCompObj.PolishArray=fstack;
		
		coCompObj.errorMes="";
		
		coCompObj.errorStatus=0;
		
		return coCompObj;		
		
		}


}

private function conUnary(s:String):String {
	
	var ts:String=s;
	
	var i:Number;
	
	var iniLen=s.length;
	
	for(i=1;i<iniLen;i++){
		
		if(ts.charAt(i)=="-"){
			
			ts=ts.substring(0,i)+"N"+ts.substring(i+1,iniLen)
			
		}
		
	}
	
	return ts;
	
}


private function checkToks(s:String):Void {
	
	var i:Number=0;
	
	var tok:String="";
	
	var ftok:String="";
	
	var sconnects:String=">+*=";
	
	var iniLen:Number=s.length;
	
	for(i=0;i<iniLen-1;i++){
		
		tok=s.charAt(i);
		
		ftok=s.charAt(i+1);
		
		if(tok=="-" && !(ftok=="(")){
															
			callError("Error after a negation connective.");
			
			return;
					 }
					 
		if(isVar(tok) && !(ftok==")") && !(sconnects.indexOf(ftok)>-1)){
			
			callError("Error after one of the variables.");
			
			return;
					 }
					 
		if(sconnects.indexOf(tok)>-1 && !(ftok=="(") && !(isVar(ftok)) && !(ftok=="-")){
			
			callError("Error after one of the connectives.");
			
			return;
					 }
					 
		if(tok=="(" && !(isVar(ftok)) && !(ftok=="-") && !(ftok=="(")){
			
			callError("Error after one of the parenthesis ( or at the beginning of your expression.");
			
			return;
					 }
					 
		if(tok==")" && !(ftok==")") && !(sconnects.indexOf(ftok)>-1)){
			
			callError("Error after one of the parenthesis ).");
			
			return;
					 }
		
		
	}
	
	
}

private function fixNotExpr(s:String):String {
	
	var pos:Number=0;
	
	var iniLen=s.length;
	
	var counter:Number=0;
	
	var ts:String=s;
	
	var ms:String=s;
	
	while(ts.indexOf("--")>-1 && counter<=iniLen){
		
		counter+=1;
		
		pos=ts.indexOf("--");
		
		ts=ts.substring(0,pos)+reduceNotExpr(ts.substring(pos,ts.length));
																		   
		
	}
	
	counter=0;
	
	pos=0;
	
	if(ts.indexOf("-")==-1){return ts;}
	
	while(counter<=iniLen){
		
		counter+=1;
		
		ms=ts.substring(pos,ts.length);
		
		pos=ms.indexOf("-")+pos;
		
		if(!isVar(ts.charAt(pos+1)) && !(ts.charAt(pos+1)=="(")){
															 
		    callError("Error after one of the negation operators.");
					  
					  break;
															 
							}
		
													   
		if(isVar(ts.charAt(pos+1))){
			
			ts=ts.substring(0,pos+1)+"("+ts.charAt(pos+1)+")"+ts.substring(pos+2,ts.length);
			
			
		} 
		
		if(ts.charAt(pos+1)=="("){
							   
					pos+=1;
					
					continue;
							   
							   }
							   
		pos=+1;
												   		
	}
	
	return ts;
	
}

private function reduceNotExpr(str:String):String {
	
	var ints:String=str;
	
	var count:Number=0;
	
	var i:Number;
	
	if(ints==undefined || ints=="" || !(ints.charAt(0)=="-")){
		
		return ints;
		
	}
	
	for(i=0; i<str.length; i++){
		
		if(ints.charAt(i)!="-"){break;}
		
		if(ints.charAt(i)=="-"){count+=1;}
		
	}
	
	if((count % 2)==0){
		
		ints=ints.substring(count,ints.length);
		
	} else {
		
		ints=ints.substring(count-1,ints.length);
	}
	
	return ints;
}

private function checkImpliesExpr(s:String):Void {
	
	var i:Number;
	
	var pos:Number=0;
	
	var iniLen=s.length;
	
	var counter:Number=0;
	
	
    for(i=0;i<iniLen;i++){
		
		if(s.charAt(i)==">"){
			
			counter+=1;
			
			pos=goRight(s,i);
			
			if(iserror==1){
				
				callError("Error at the implication number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)==")")){
						
			    callError("Ambiguous expression. Error possibly at the implication number "+String(counter)+".");
				
				return;
		}
				
			pos=goLeft(s,i);
			
			if(iserror==1){
				
				callError("Error at the implication number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)=="(")){
						
			    callError("Ambiguous expression. Error possibly at the implication number "+String(counter)+".");
				
				return;
		}
				
						
		}
			
	}
	
}


private function checkIffExpr(s:String):Void {
	
	var i:Number;
	
	var pos:Number=0;
	
	var iniLen=s.length;
	
	var counter:Number=0;
		
    for(i=0;i<iniLen;i++){
		
		if(s.charAt(i)=="="){
			
			counter+=1;
			
			pos=goRight(s,i);
			
			if(iserror==1){
				
				callError("Error at the biconditional number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)==")")){
						
			    callError("Ambiguous expression. Error possibly at the biconditional number "+String(counter)+".");
				
				return;
		}
				
			pos=goLeft(s,i);
			
			if(iserror==1){
				
				callError("Error at the biconditional number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)=="(")){
						
			    callError("Ambiguous expression. Error possibly at the biconditional number "+String(counter)+".");
				
				return;
		}
				
						
		}
			
	}
	
	
	
}


private function checkAndExpr(s:String):Void {
	
	var i:Number;
	
	var pos:Number=0;
	
	var iniLen=s.length;
	
	var counter:Number=0;
		
    for(i=0;i<iniLen;i++){
		
		if(s.charAt(i)=="*"){
			
			counter+=1;
			
			pos=goRightCheck(s,i,"*");
			
			if(iserror==1){
				
				callError("Error at the conjuction number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)==")")){
						
			    callError("Ambiguous expression. Error possibly at the conjuction number "+String(counter)+".");
				
				return;
		}
				
			pos=goLeftCheck(s,i,"*");
			
			if(iserror==1){
				
				callError("Error at the conjuction number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)=="(")){
						
			    callError("Ambiguous expression. Error possibly at the conjuction number "+String(counter)+".");
				
				return;
		}
				
						
		}
			
	}
	
	
	
}


private function checkOrExpr(s:String):Void {
	
	var i:Number;
	
	var pos:Number=0;
	
	var iniLen=s.length;
	
	var counter:Number=0;
		
    for(i=0;i<iniLen;i++){
		
		if(s.charAt(i)=="+"){
			
			counter+=1;
			
			pos=goRightCheck(s,i,"+");
			
			if(iserror==1){
				
				callError("Error at the disjuction number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)==")")){
						
			    callError("Ambiguous expression. Error possibly at the disjuction number "+String(counter)+".");
				
				return;
		}
				
			pos=goLeftCheck(s,i,"+");
			
			if(iserror==1){
				
				callError("Error at the disjuction number "+String(counter)+".");
				
				return;
				
			}
			
			if(!(s.charAt(pos)=="(")){
						
			    callError("Ambiguous expression. Error possibly at the disjuction number "+String(counter)+".");
				
				return;
		}
				
						
		}
			
	}
	
	
	
}


private function goRightCheck(s:String,p:Number,ch:String):Number {
	
	var rch:String=ch;
	
	var rpos:Number=p;
	
	var len:Number=s.length;
	
	var count:Number=0;
	
	while(rch==ch && count<len){
		
		count+=1;
			
		rpos=goRight(s,rpos);
		
		if(iserror==1){break;}
		
		rch=s.charAt(rpos);
		
	}
	
	return rpos;
	
	
}


private function goLeftCheck(s:String,p:Number,ch:String):Number {
	
	var lch:String=ch;
	
	var lpos:Number=p;
	
	var len:Number=s.length;
	
	var count:Number=0;
	
	while(lch==ch && count<len){
		
		count+=1;
			
		lpos=goLeft(s,lpos);
		
		if(iserror==1){break;}
		
		lch=s.charAt(lpos);
		
	}
	
	return lpos;
	
	
}



private function changeNotation(s:String):String {
	
	var counter:Number=0;
	
	var pos:Number=0;
	
	var ts:String=s;
	
	var iniLen:Number=s.length;
	
	
	while(ts.indexOf("not")>-1 && counter<iniLen){
		
		pos=ts.indexOf("not");
		
		ts=ts.substring(0,pos)+"-"+ts.substring(pos+3);
		
		counter+=1;
	
	}
	
	counter=0;
	
	while(ts.indexOf("implies")>-1 && counter<iniLen){
		
		pos=ts.indexOf("implies");
		
		ts=ts.substring(0,pos)+">"+ts.substring(pos+7);
		
		counter+=1;
		
	}
	
	counter=0;
	
	while(ts.indexOf("and")>-1 && counter<iniLen){
		
		pos=ts.indexOf("and");
		
		ts=ts.substring(0,pos)+"*"+ts.substring(pos+3);
		
		counter+=1;
	
	}
	
	counter=0;
	
	while(ts.indexOf("or")>-1 && counter<iniLen){
		
		pos=ts.indexOf("or");
		
		ts=ts.substring(0,pos)+"+"+ts.substring(pos+2);
		
		counter+=1;
		
	}
	
	counter=0;
	
	while(ts.indexOf("iff")>-1 && counter<iniLen){
		
		pos=ts.indexOf("iff");
		
		ts=ts.substring(0,pos)+"="+ts.substring(pos+3);
		
		counter+=1;
	
	}
	
	counter=0;
	
	ts="("+ts+")";
  
	return ts;
	
	
}


private function whiteSpaces(inputstring:String):String {

   var curpos, transtring, inilen, counter=0;
   
   inilen=inputstring.length;
   
   transtring=inputstring.toLowerCase();
   
   while(transtring.indexOf(" ")>-1 && counter < inilen+1){
	   
	 curpos=transtring.indexOf(" ");
	 
	 transtring=transtring.substring(0,curpos) + 
	 transtring.substring(curpos+1,transtring.length);
	   
	 counter+=1;
	
   }

   return transtring;
		
}

private function checkLegal(inputstring:String):Boolean {
	
	  var i, legal, curchar;
  
      for(i=0; i<inputstring.length; i++){
	
      curchar=inputstring.charAt(i);
	
      legal=lets.indexOf(curchar)+opers.indexOf(curchar);

      if(legal==-2){
	   
	      callError("Unknown characters. Only letters and paretheses allowed.");
	 
	      return false;}
	 
   }
      return true;	
}


private function checkLegalTwo(inputstring:String):Boolean {
	
	  var i, legal, curchar;
  
      for(i=0; i<inputstring.length; i++){
	
      curchar=inputstring.charAt(i);
	
      legal=connects.indexOf(curchar)+opers.indexOf(curchar);

      if(!isVar(curchar) && legal==-2){
	   
	      callError("Unknown characters. Check your variables' names and your spelling.");
	 
	      return false;
		  
		  }
	 
   }
      return true;	
}


private function callError(mes:String):Void {
      
	  errorMes="Syntax error. " + mes;
	  
	  iserror=1;
	
}



private function checkPars(inputstring:String):Boolean {
	
	var i, j, matchpar, left=0, right=0, counter=0;
	
	for(i=0; i<inputstring.length; i++){
		
		if(inputstring.charAt(i)=="("){left+=1;}
									
		if(inputstring.charAt(i)==")"){right+=1;}
	}
		
		if(!(left==right)){
			
			callError("Mismatched parenthesis.");
			
			return false;}
			
	for(j=0; j<inputstring.length; j++){
			
		if(inputstring.charAt(j)=="("){
									
	    matchpar=1; counter=0;
			
        while(matchpar>0 && counter<inputstring.length)
		   {  
		      counter+=1;
			  
			  if(inputstring.charAt(j+counter)=="("){matchpar+=1;}
													
		      if(inputstring.charAt(j+counter)==")"){matchpar+=-1;}
			  
		   }
		  
		   if(matchpar>0){
			   
			j+=1;
		    
			callError("Mismatched parenthesis."); 
			
			return false;
			
			}
			
		  }
					
		}
		
		
		for(j=0; j<inputstring.length; j++){
			
		if(inputstring.charAt(j)==")"){
					
	    matchpar=1; counter=0;
			
        while(matchpar>0 && counter<inputstring.length)
		   {  
		      counter+=1;
			  
			  if(inputstring.charAt(j-counter)==")"){matchpar+=1;}
													
		      if(inputstring.charAt(j-counter)=="("){matchpar+=-1;}
			  
		   }
		   
	
		   if(matchpar>0){
			   
			j+=1;
		    
			callError("Mismatched parenthesis."); 
			
			return false;
			
			}
			
		  }
					
		}
			
		return true;
}



private function goRight(inputstring:String, pos:Number):Number {
	
	var rightchar, rightcounter, matchpar;
	
	rightchar=inputstring.charAt(pos+1);
	
	rightcounter=pos+1;
	
     if(rightchar=="-"){
	   
	    rightcounter+=1; 
	   
        if(rightcounter>=inputstring.length){
	    
		iserror=1;
		
		return rightcounter;
		
		   }
		
		else {
			
	    rightchar=inputstring.charAt(rightcounter);
		
		        }
				
          }
  

		  
     if(isVar(rightchar)){
		 
		    rightcounter+=1;
   	   		
			return rightcounter;
			
		  }
		  
		  
    else if(rightchar=="("){
			
	  matchpar=1;
			
      while(matchpar>0 && rightcounter<inputstring.length)
		   {  
		      rightcounter+=1;
		   
			  if(inputstring.charAt(rightcounter)=="("){matchpar+=1;}
													
		      if(inputstring.charAt(rightcounter)==")"){matchpar+=-1;}
			  
		   }
		   
		   rightcounter+=1;
		      
		   if(matchpar>0){
		    
			iserror=1;
			
			return rightcounter;}
			
		           }
		  
    else { 
			iserror=1;
			  
			return rightcounter;
			
			}
			
		return rightcounter;
			  
}


private function goRightCon(inputstring:String, pos:Number):Number {
	
	var rightchar, rightcounter, matchpar;
	
	rightchar=inputstring.charAt(pos+1);
	
	rightcounter=pos+1;
	
	if(!(isVar(rightchar)) && funs.indexOf(rightchar)==-1 && !(rightchar=="(")){
																			
					iserror=1;
		
		            return rightcounter;																			
																			
					}
	
	if(isVar(rightchar)){
		 
		    rightcounter+=1;
   	   		
			return rightcounter;
			
		  }
	
     if(funs.indexOf(rightchar)>-1){
	   
	    rightcounter+=1; 
	   
        if(rightcounter>=inputstring.length){
	    
		iserror=1;
		
		return rightcounter;
		
		   }
		
		else {
			
	    rightchar=inputstring.charAt(rightcounter);
		
		if(!(rightchar=="(")){
				
				iserror=1;
		
		        return rightcounter;
						  
						  
						  }
		
		        }
				
          }
  	  
		  
    if(rightchar=="("){
			
	  matchpar=1;
			
      while(matchpar>0 && rightcounter<inputstring.length)
		   {  
		      rightcounter+=1;
		   
			  if(inputstring.charAt(rightcounter)=="("){matchpar+=1;}
													
		      if(inputstring.charAt(rightcounter)==")"){matchpar+=-1;}
			  
		   }
		   
		   rightcounter+=1;
		      
		   if(matchpar>0){
		    
			iserror=1;
			
			return rightcounter;}
			
		           }
		  
  	
		return rightcounter;
			  
}



private function goLeft(inputstring:String, pos:Number):Number {
	
	var leftchar, leftcounter, matchpar;
	
	leftchar=inputstring.charAt(pos-1);
	
	leftcounter=pos-1;
	
	if(leftcounter<=0){
		
		    iserror=1;
			
			return leftcounter;
		
	}
	  
     if(isVar(leftchar)){
		 
		    leftcounter+=-1;
			
			if(inputstring.charAt(leftcounter)=="-"){
				
				leftcounter+=-1;
				
			}
   	   		
			return leftcounter;
			
		  }
		  
		  
    else if(leftchar==")"){
			
	  matchpar=1;
	  			
      while(matchpar>0 && leftcounter>0)
		   {  
		      leftcounter+=-1;
		   
			  if(inputstring.charAt(leftcounter)==")"){matchpar+=1;}
													
		      if(inputstring.charAt(leftcounter)=="("){matchpar+=-1;}
			  
		   }
		   
		   leftcounter+=-1;
		      
		   if(matchpar>0){
		    
			iserror=1;
			
			return leftcounter;
			
			    }
				
			if(inputstring.charAt(leftcounter)=="-"){
				
				leftcounter+=-1;
				
			}
			
		           }
		  
    else { 
			iserror=1;
			  
			return leftcounter;
			
			}
			
		return leftcounter;
			  
}



private function goLeftCon(inputstring:String, pos:Number):Number {
	
	var leftchar, leftcounter, matchpar;
	
	leftchar=inputstring.charAt(pos-1);
	
	leftcounter=pos-1;
	
	if(leftcounter<=0){
		
		    iserror=1;
			
			return leftcounter;
		
	}
	  
     if(isVar(leftchar)){
		 
		    leftcounter+=-1;
			
			return leftcounter;
			
		  }
		  
		  
    else if(leftchar==")"){
			
	  matchpar=1;
	  			
      while(matchpar>0 && leftcounter>0)
		   {  
		      leftcounter+=-1;
		   
			  if(inputstring.charAt(leftcounter)==")"){matchpar+=1;}
													
		      if(inputstring.charAt(leftcounter)=="("){matchpar+=-1;}
			  
		   }
		   
		   leftcounter+=-1;
		      
		   if(matchpar>0){
		    
			iserror=1;
			
			return leftcounter;
			
			    }
				
			leftchar=inputstring.charAt(leftcounter);
				
			if(funs.indexOf(leftchar)>-1){
				
				leftcounter+=-1;
				
			}
			
		           }
		  
    else { 
			iserror=1;
			  
			return leftcounter;
			
			}
			
		return leftcounter;
			  
}


private function conOper(inputstring:String, ch:String):String {

  var transtring, inilen, mco, curpos, leftoper, rightoper,leftmove,rightmove;

  inilen=inputstring.length;
  
  transtring=inputstring;
  
  if(transtring.indexOf(ch)==-1){
		
        return transtring;
		
		}
					
   mco=0;
			
   while(transtring.indexOf(ch)>0 && mco<inilen*6){
	   
	   mco+=1;
   
	   curpos=transtring.indexOf(ch);
	 
	   leftmove=goLeftCon(transtring,curpos);
	   
	   if(iserror==1){
		   
		   callError("");
		   
		   return ""; } 
		   
		else { leftoper=transtring.substring(leftmove+1,curpos); }
		
		rightmove=goRightCon(transtring,curpos);
	   
	   if(iserror==1){
		   
		   callError("");
		   
		   return ""; } 
		   
		else { rightoper=transtring.substring(curpos+1, rightmove); }
		
		if(ch=="*"){
		
	   transtring=transtring.substring(0,leftmove+1)
	           +"A("+ leftoper+ "," + rightoper + ")" + 
	   transtring.substring(rightmove,transtring.length+1);
		}
		
		if(ch=="+"){
		
	   transtring=transtring.substring(0,leftmove+1)
	           +"O("+ leftoper+ "," + rightoper + ")" + 
	   transtring.substring(rightmove,transtring.length+1);
		}
		
		if(ch==">"){
		
	   transtring=transtring.substring(0,leftmove+1)
	           +"I("+ leftoper + "," + rightoper + ")" + 
	   transtring.substring(rightmove,transtring.length+1);
		}
		
		if(ch=="="){
		
	   transtring=transtring.substring(0,leftmove+1)
	           +"B("+ leftoper + "," + rightoper + ")" + 
	   transtring.substring(rightmove,transtring.length+1);
		}
	   
	   if(transtring.length>inilen*7){
		 
		 callError("Oooops!");
		 
		 return "";}
}
		
	return transtring;
   
		
}


private function istokf1(chars:String):Boolean {
	
	var i:Number;
	
	for(i=0; i< f1Array.length; i++){
		
	  if(chars == f1Array[i]){return true;}
	  
	}
	
	return false;
}


private function isVar(chars:String):Boolean {
	
	var i:Number;
	
	for(i=0; i< aVarNames.length; i++){
		
	  if(chars == aVarNames[i]){return true;}
	  
	}
	
	return false;
}


private function istokf2(chars:String):Boolean {
	
	var i:Number;
	
	for(i=0; i< f2Array.length; i++){
		
	  if(chars == f2Array[i]){return true;}
	  
	}
	
	return false;
}




private function setToken(curtype:String,curvalue:String,curlength:Number):Void {
	
	tokentype=curtype;
	
	tokenvalue=curvalue;
	
	tokenlength=curlength;
}


private function nextToken(s:String,pos:Number):Boolean {
	
  var ch:String;
  
  var inilen:Number;
  
  inilen=s.length;
 
  if(pos >= inilen || pos<0){return false; }
     
        ch=s.charAt(pos);
		
		if(funs.indexOf(ch)>-1){
			
		  if(istokf1(ch)){setToken("f1",ch, 1); return true; }
		  
		  else {setToken("f2",ch, 1); return true;}
									 	
		              }
					  	
		 
		  if(opers.indexOf(ch)>-1 || ch==","){ 
		    
			   setToken("oper",ch,1); return true;
		  }
		  
		   if(isVar(ch)){ 
		    
			   setToken("v",ch,1); return true;
		  }
		  
		  
		  return false;
  		
}


private function makeStack(inputstring:String):Array {
	
	var mstring, minilen, mpos, mstack, counter;
		
	mstring=inputstring;
	
	mpos=0;
	
	mstack=[];
	
	minilen=mstring.length;
	
	counter=0;
	

while(mpos < minilen && counter<minilen*2){
		
	if(nextToken(mstring,mpos)){
	
	       if(tokentype=="f1"){
			
			 mstack.push(this[tokenvalue]);
			 
			 mstack.push("f1");
			 
		   }
		   
		   if(tokentype=="f2"){
			
			 mstack.push(this[tokenvalue]);
			 
			 mstack.push("f2");
			
		   }
		   
		   if(tokentype=="v"){
			
			 mstack.push(tokenvalue);
			 
			 mstack.push("v");
			
		   }
		   
	}
	
	else {callError("Unknown characters."); return [];}
	
	mpos+=tokenlength;
	
	counter+=1;

}


mstack.reverse();


return(mstack);
	
}


private function O(a,b):Number{
	
	if((a+b)>0){return 1;} else {return 0;}
	
}


private function A(a,b):Number{
	
	if((a+b)==2){return 1;} else {return 0;}
	
}

private function I(a,b):Number{
	
	if(a==1 && b==0){return 0;} else {return 1;}
	
}

private function B(a,b):Number{
	
	if(a==b){return 1;} else {return 0;}
	
}

private function N(a):Number{
	
	if(a==1){return 0;} else {return 1;}
	
}



public function doEval(compiledExpression:Array,aVarVals:Array): Number {

	var entrytype:String = "";

	var operands:Array = [];
	
	var i:Number;
	
	var j:Number;
	
	var arg1;
	
	var arg2;
	
	if(aVarVals.length!=aVarNames.length){
		
		return NaN; 
		
	}
	
	for(j = 0; j < aVarVals.length; j++){
		
		if(aVarVals[j]!=0 && aVarVals[j]!=1){
			
			return NaN;
			
		}
		
	}

	for(i = 0; i < compiledExpression.length; i++){

		entrytype = compiledExpression[i++];
       
	    if( entrytype == "v"){
						
		for(j=0; j<aVarNames.length; j++){
			
			if(compiledExpression[i]==aVarNames[j]){
				
				operands.push(aVarVals[j]);
				
			}
			
		}
				
		}  
		
		else if( entrytype == "f1"){
			
			operands.push(compiledExpression[i](operands.pop()))				

		} 
		
		else if( entrytype == "f2"){
			
			arg1=operands.pop(); arg2=operands.pop();
			
			operands.push(compiledExpression[i](arg1,arg2));				

		} 
		
		else {

			return NaN;

		}
		
	}
	
	return operands[0]

}

}
